-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8

if not RaidChecklist then return end

local L = RaidChecklistStrings
local P = RaidChecklistPetFamilies

RaidChecklistData = {}
local data = RaidChecklistData

--
-- Primary ability categories
--
data.buffs = {
  "CRIT", "ATTACKPWR",
  "SPELLPWR",
  "STATS", "STAM", "MASTERY",
  "HASTE", "MULTI", "VERSA",
}

data.support = {
 "BLOODLUST", "COMBATRES", "BURSTMAGICPROT"
}

data.dispels = {
  "OFFMAGIC", "DEFMAGIC",
  "DISEASE", "POISON",
  "CURSE", "ENRAGE"
}

data.ccs = {
  "CCBEASTS", "CCDEMONS", "CCDRAGONKIN", "CCELEMENTALS", "CCGIANTS", "CCHUMANOIDS", "CCUNDEAD", "CCANY",
  "FEARBEASTS", "FEARUNDEAD", "FEARANY"
}

data.mcs = {
  "KNOCKBACK", "SNARE", "AOESNARE", "ROOT", "STUN", "AOESTUN"
}

data.miscs = {
  "MORTALSTRIKE",
  "INTERRUPT", "INTERRUPT2"
}


--
-- Secondary ability categories.
--
-- All abilities in these categories here are already covered by the primary categories.
-- These groupings exist purely for display groupings.
--

data.hunterpetabilities = {
  "CRIT", "STATS", "STAM", "SPELLPWR",
  "HASTE", "MASTERY", "BLOODLUST", "COMBATRES",
  "MORTALSTRIKE", "MULTI", "VERSA",
  "INTERRUPT", "INTERRUPT2", "STUN",
}

data.warlockpetabilities = {
  "DEFMAGIC",
  "MANA", "MANAREGEN",
  "MORTALSTRIKE",
  "CCHUMANOIDS",
  "KNOCKBACK", "INTERRUPT"
}


-- Race/class combos
L.BLOODELF_ROGUE   = L.BLOODELF.." "..L.ROGUE.."|r"
local SEP = "|r/"
L.BLOODELF_CASTER  = L.BLOODELF.." "..L.MAGE..SEP..L.PALADIN..SEP..L.PRIEST..SEP..L.WARLOCK.."|r"
L.BLOODELF_WARRIOR = L.BLOODELF.." "..L.WARRIOR.."|r"
L.BLOODELF_DEATHKNIGHT = L.BLOODELF.." "..L.DEATHKNIGHT.."|r"
L.BLOODELF_HUNTER = L.BLOODELF.." "..L.HUNTER.."|r"
L.BLOODELF_MONK = L.BLOODELF.." "..L.MONK.."|r"
L.RACIAL = select (2, GetSpellInfo (25046))


-- Abuse the metatable functionality in order to make sure any abilities referenced by the providers actually exists.
local ability_typo_checker = { __index = function (table, key)
  for i,b in ipairs (table) do
    if b == key then return key end
  end
  print ("|cffff0000>> RaidChecklist: unknown ability name: |r"..(key or "nil"))
end
}
setmetatable (data.buffs, ability_typo_checker)
setmetatable (data.support, ability_typo_checker)
setmetatable (data.dispels, ability_typo_checker)
setmetatable (data.ccs, ability_typo_checker)
setmetatable (data.mcs, ability_typo_checker)
setmetatable (data.miscs, ability_typo_checker)

-- Convenience function to create a "provider" table
data.create_provider_table = function (self)
  local t = {}
  for i,b in ipairs (self.buffs)   do t[b] = {} end
  for i,b in ipairs (self.support) do t[b] = {} end
  for i,b in ipairs (self.dispels) do t[b] = {} end
  for i,b in ipairs (self.ccs)     do t[b] = {} end
  for i,b in ipairs (self.mcs)     do t[b] = {} end
  for i,b in ipairs (self.miscs)   do t[b] = {} end
  return t
end


--
-- Core ability table.
--

local B = data.buffs
local T = data.support
local S = data.dispels
local C = data.ccs
local O = data.mcs
local M = data.miscs

local R = { BLOODELF = "BloodElf" } -- "Filename" from UnitRace(), locale-independent

local SPEC = " "

-- Legend:
--   a - Ability
--   c - Class (localized)
--   s - Spec name (localized)
--   p - Pet family (localized)
--   l - Level requirement
--   t - Talent ID
--   d - Detailed info: table with spell_id/talent_id/perk_id/icon_id to show in detailed view { s = X, t = Y, p = Z, i = V }
--   e - Race-specific ability
--   g - glyph-id (ability, not create-glyph spell)
--   v - true if passive ability
--   display - Text to display for this entry
data.table = {
  { a = S.CURSE,          c = L.DRUID,        s = L.BALANCE,                          l = 22,                d = { s = 2782                         }                                                },
  { a = S.CURSE,          c = L.DRUID,        s = L.FERAL,                            l = 22,                d = { s = 2782                         }                                                },
  { a = S.CURSE,          c = L.DRUID,        s = L.GUARDIAN,                         l = 22,                d = { s = 2782                         }                                                },
  { a = S.CURSE,          c = L.DRUID,        s = L.RESTODRUID,                       l = 22,                d = { s = 88423                        }                                                },
  { a = S.CURSE,          c = L.MAGE,                                                 l = 29,                d = { s = 475                          }                                                },
  { a = S.CURSE,          c = L.SHAMAN,                                               l = 18,                d = { s = 51886                        }                                                },
  { a = S.CURSE,          c = L.SHAMAN,       s = L.RESTOSHAMAN,                      l = 18,                d = { s = 77130                        }                                                },

  { a = S.DISEASE,        c = L.MONK,                                                 l = 20,                d = { s = 115450                       }                                                },
  { a = S.DISEASE,        c = L.MONK,         s = L.MISTWEAVER,                       l = 78,                d = { s = 115310                       }                                                },
  { a = S.DISEASE,        c = L.PALADIN,                                              l = 20,                d = { s = 4987                         }                                                },
  { a = S.DISEASE,        c = L.PRIEST,       s = L.HOLYPRIEST,                       l = 22,                d = { s = 527                          }                                                },
  { a = S.DISEASE,        c = L.PRIEST,       s = L.DISCIPLINE,                       l = 22,                d = { s = 527                          }                                                },

  { a = S.ENRAGE,         c = L.DRUID,                                                l = 60,                d = { s = 2908                         }                                                },
  { a = S.ENRAGE,         c = L.HUNTER,                                               l = 35,                d = { s = 19801                        }                                                },
  { a = S.ENRAGE,         c = L.ROGUE,                                                l = 70,                d = { s = 5938                         }                                                },

  { a = S.DEFMAGIC,       c = L.DRUID,        s = L.RESTODRUID,                       l = 22,                d = { s = 88423                        }                                                },
  { a = S.DEFMAGIC,       c = L.MONK,         s = L.MISTWEAVER,                       l = 20,                d = { s = 115451                       }                                                },
  { a = S.DEFMAGIC,       c = L.MONK,         s = L.MISTWEAVER,                       l = 78,                d = { s = 115310                       }                                                },
  { a = S.DEFMAGIC,       c = L.PALADIN,      s = L.HOLYPALADIN,                      l = 20,                d = { s = 53551                        }                                                },
  { a = S.DEFMAGIC,       c = L.PRIEST,       s = L.HOLYPRIEST,                       l = 22,                d = { s = 527                          }                                                },
  { a = S.DEFMAGIC,       c = L.PRIEST,       s = L.DISCIPLINE,                       l = 22,                d = { s = 527                          }                                                },
  { a = S.DEFMAGIC,       c = L.PRIEST,                                               l = 72,                d = { s = 32375                        }                                                },
  { a = S.DEFMAGIC,       c = L.SHAMAN,       s = L.RESTOSHAMAN,                      l = 18,                d = { s = 77130                        }                                                },
  { a = S.DEFMAGIC,       c = L.WARLOCK,                          p = P.IMP,                                 d = { s = 89808                        }                                                },
  { a = S.DEFMAGIC,       c = L.WARLOCK,                          p = P.FELIMP,                 t = 19293,   d = { s = 115276, t = 19293            }                                                },--Talentpet

  { a = S.OFFMAGIC,       c = L.DEATHKNIGHT,                                                                 d = { s = 45477,  g = 58631            },                                   g = 58631   },
  { a = S.OFFMAGIC,       c = L.HUNTER,                                               l = 35,                d = { s = 19801                        }                                                },
  { a = S.OFFMAGIC,       c = L.MAGE,                                                 l = 64,                d = { s = 30449                        }                                                },
  { a = S.OFFMAGIC,       c = L.PRIEST,                                               l = 26,                d = { s = 528                          }                                                },
  { a = S.OFFMAGIC,       c = L.PRIEST,                                               l = 72,                d = { s = 32375                        }                                                },
  { a = S.OFFMAGIC,       c = L.SHAMAN,                                               l = 12,                d = { s = 370                          }                                                },
  { a = S.OFFMAGIC,       c = L.WARLOCK,                          p = P.OBSERVER,               t = 19293,   d = { s = 115284, t = 19293            }                                                },--Talentpet

  { a = S.POISON,         c = L.DRUID,        s = L.BALANCE,                          l = 22,                d = { s = 2782                         }                                                },
  { a = S.POISON,         c = L.DRUID,        s = L.FERAL,                            l = 22,                d = { s = 2782                         }                                                },
  { a = S.POISON,         c = L.DRUID,        s = L.GUARDIAN,                         l = 22,                d = { s = 2782                         }                                                },
  { a = S.POISON,         c = L.DRUID,        s = L.RESTODRUID,                       l = 22,                d = { s = 88423                        }                                                },
  { a = S.POISON,         c = L.MONK,                                                 l = 20,                d = { s = 115450                       }                                                },
  { a = S.POISON,         c = L.MONK,         s = L.MISTWEAVER,                       l = 78,                d = { s = 115310                       }                                                },
  { a = S.POISON,         c = L.PALADIN,                                              l = 20,                d = { s = 4987                         }                                                },

  { a = B.STAM,           c = L.HUNTER,                           p = P.BEAR,                                d = { s = 50256                        }                                                },
  { a = B.STAM,           c = L.HUNTER,                           p = P.GOAT,                                d = { s = 160014                       }                                                },
  { a = B.STAM,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.RYLAK,                               d = { s = 160003                       }                                                },
  { a = B.STAM,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SILITHID,                            d = { s = 90364                        }                                                },
  { a = B.STAM,           c = L.PRIEST,                                               l = 22,                d = { s = 21562                        }                                                },
  { a = B.STAM,           c = L.WARLOCK,                                              l = 60,                d = { s = 166928                       }, v = true                                      },
  { a = B.STAM,           c = L.WARRIOR,                                              l = 68,                d = { s = 469                          }                                                },

  { a = T.BLOODLUST,      c = L.HUNTER,                           p = P.NETHERRAY,                           d = { s = 160452                       }                                                },
  { a = T.BLOODLUST,      c = L.HUNTER,       s = L.BEASTMASTER,  p = P.COREHOUND,                           d = { s = 90355                        }                                                },
  { a = T.BLOODLUST,      c = L.MAGE,                                                 l = 84,                d = { s = 80353                        }                                                },
  { a = T.BLOODLUST,      c = L.SHAMAN,                                               l = 70,                d = { s = 2825                         }                                                },
  { a = T.BLOODLUST,      c = L.SHAMAN,                                               l = 70,                d = { s = 32182                        }                                                },

  { a = T.COMBATRES,      c = L.DEATHKNIGHT,                                          l = 72,                d = { s = 61999                        }                                                },
  { a = T.COMBATRES,      c = L.DRUID,                                                l = 56,                d = { s = 20484                        }                                                },
  { a = T.COMBATRES,      c = L.HUNTER,                           p = P.CRANE,                               d = { s = 159931                       }                                                },
  { a = T.COMBATRES,      c = L.HUNTER,                           p = P.MOTH,                                d = { s = 159956                       }                                                },
  { a = T.COMBATRES,      c = L.HUNTER,       s = L.BEASTMASTER,  p = P.QUILEN,                              d = { s = 126393                       }                                                },
  { a = T.COMBATRES,      c = L.WARLOCK,                                              l = 18,                d = { s = 20707                        }                                                },

  { a = T.BURSTMAGICPROT, c = L.DEATHKNIGHT,                                          l = 58,   t = 19219,   d = {             t = 19219, i = 51052 }                                                },
  { a = T.BURSTMAGICPROT, c = L.WARRIOR,                                              l = 75,   t = 15765,   d = {             t = 15765, i = 114028}                                                },

  { a = B.ATTACKPWR,      c = L.DEATHKNIGHT,                                          l = 65,                d = { s = 57330                        }                                                },
  { a = B.ATTACKPWR,      c = L.HUNTER,                                               l = 39,                d = { s = 19506                        }, v = true                                      },
  { a = B.ATTACKPWR,      c = L.WARRIOR,                                              l = 42,                d = { s = 6673                         }                                                },

  { a = B.CRIT,           c = L.DRUID,        s = L.FERAL,                            l = 46,                d = { s = 17007                        }, v = true                                      },
  { a = B.CRIT,           c = L.HUNTER,       s = L.MARKSMAN,                         l = 100,  t = 21197,   d = { s = 160200, t = 21197            }, v = true                                      },
  { a = B.CRIT,           c = L.HUNTER,       s = L.SURVIVAL,                         l = 100,  t = 21197,   d = { s = 160200, t = 21197            }, v = true                                      },
  { a = B.CRIT,           c = L.HUNTER,                           p = P.RAPTOR,                              d = { s = 160052                       }                                                },
  { a = B.CRIT,           c = L.HUNTER,                           p = P.WOLF,                                d = { s = 24604                        }                                                },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.DEVILSAUR,                           d = { s = 90309                        }                                                },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.QUILEN,                              d = { s = 126373                       }                                                },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WATERSTRIDER,                        d = { s = 126309                       }, v = true                                      },
  { a = B.CRIT,           c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SHALESPIDER,                         d = { s = 90363                        }                                                },
  { a = B.CRIT,           c = L.MAGE,                                                 l = 58,                d = { s = 1459                         }                                                },
  { a = B.CRIT,           c = L.MONK,         s = L.BREWMASTER,                       l = 81,                d = { s = 116781                       }                                                },
  { a = B.CRIT,           c = L.MONK,         s = L.WINDWALKER,                       l = 81,                d = { s = 116781                       }                                                },

  { a = B.HASTE,          c = L.DEATHKNIGHT,  s = L.FROSTDK,                          l = 60,                d = { s = 55610                        }, v = true                                      },
  { a = B.HASTE,          c = L.DEATHKNIGHT,  s = L.UNHOLY,                           l = 60,                d = { s = 55610                        }, v = true                                      },
  { a = B.HASTE,          c = L.ROGUE,                                                l = 30,                d = { s = 113742                       }, v = true                                      },
  { a = B.HASTE,          c = L.HUNTER,       s = L.MARKSMAN,                         l = 100,  t = 21197,   d = { s = 160203, t = 21197            }, v = true                                      },
  { a = B.HASTE,          c = L.HUNTER,       s = L.SURVIVAL,                         l = 100,  t = 21197,   d = { s = 160203, t = 21197            }, v = true                                      },
  { a = B.HASTE,          c = L.HUNTER,                           p = P.HYENA,                               d = { s = 128432                       }                                                },
  { a = B.HASTE,          c = L.HUNTER,                           p = P.SPOREBAT,                            d = { s = 135678                       }                                                },
  { a = B.HASTE,          c = L.HUNTER,                           p = P.WASP,                                d = { s = 160074                       }                                                },
  { a = B.HASTE,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.RYLAK,                               d = { s = 160003                       }                                                },
  { a = B.HASTE,          c = L.SHAMAN,                                               l = 80,                d = { s = 116956                       }, v = true                                      },

  -- multistrike
  { a = B.MULTI,          c = L.HUNTER,                           p = P.BAT,                                 d = { s = 50519                        }                                                },
  { a = B.MULTI,          c = L.HUNTER,                           p = P.DRAGONHAWK,                          d = { s = 34889                        }                                                },
  { a = B.MULTI,          c = L.HUNTER,                           p = P.WINDSERPENT,                         d = { s = 24844                        }                                                },
  { a = B.MULTI,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.CLEFTHOOF,                           d = { s = 57386                        }                                                },
  { a = B.MULTI,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.COREHOUND,                           d = { s = 58604                        }                                                },
  { a = B.MULTI,          c = L.MONK,                                                 l = 80,                d = { s = 166916                       }, v = true                                      },
  { a = B.MULTI,          c = L.ROGUE,                                                l = 30,                d = { s = 113742                       }, v = true                                      },
  { a = B.MULTI,          c = L.WARLOCK,                                              l = 82,                d = { s = 109773                       }                                                },

  -- versatility
  { a = B.VERSA,          c = L.DEATHKNIGHT,  s = L.FROST,                            l = 60,                d = { s = 55610                        }, v = true                                      },
  { a = B.VERSA,          c = L.DEATHKNIGHT,  s = L.UNHOLY,                           l = 60,                d = { s = 55610                        }, v = true                                      },
  { a = B.VERSA,          c = L.DRUID,                                                l = 62,                d = { s = 1126                         }                                                },
  { a = B.VERSA,          c = L.HUNTER,                           p = P.BIRDOFPREY,                          d = { s = 159735                       }                                                },
  { a = B.VERSA,          c = L.HUNTER,                           p = P.BOAR,                                d = { s = 35290                        }                                                },
  { a = B.VERSA,          c = L.HUNTER,                           p = P.RAVAGER,                             d = { s = 50518                        }                                                },
  { a = B.VERSA,          c = L.HUNTER,                           p = P.PORCUPINE,                           d = { s = 160045                       }                                                },
  { a = B.VERSA,          c = L.HUNTER,                           p = P.STAG,                                d = { s = 173035                       }                                                },
  { a = B.VERSA,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.CLEFTHOOF,                           d = { s = 57386                        }                                                },
  { a = B.VERSA,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WORM,                                d = { s = 160077                       }                                                },
  { a = B.VERSA,          c = L.PALADIN,      s = L.RETRIBUTION,                      l = 80,                d = { s = 167187                       }, v = true                                      },
  { a = B.VERSA,          c = L.WARRIOR,      s = L.ARMS,                             l = 80,                d = { s = 167188                       }, v = true                                      },
  { a = B.VERSA,          c = L.WARRIOR,      s = L.FURY,                             l = 80,                d = { s = 167188                       }, v = true                                      },

  { a = B.SPELLPWR,       c = L.MAGE,                                                 l = 58,                d = { s = 1459                         }                                                },
  { a = B.SPELLPWR,       c = L.WARLOCK,                                              l = 82,                d = { s = 109773                       }                                                },
  { a = B.SPELLPWR,       c = L.HUNTER,       s = L.MARKSMAN,                         l = 100,  t = 21197,   d = { s = 160205, t = 21197            }, v = true                                      },
  { a = B.SPELLPWR,       c = L.HUNTER,       s = L.SURVIVAL,                         l = 100,  t = 21197,   d = { s = 160205, t = 21197            }, v = true                                      },
  { a = B.SPELLPWR,       c = L.HUNTER,                           p = P.SERPENT,                             d = { s = 128433                       }                                                },
  { a = B.SPELLPWR,       c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SILITHID,                            d = { s = 90364                        }                                                },
  { a = B.SPELLPWR,       c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WATERSTRIDER,                        d = { s = 126309                       }, v = true                                      },

  { a = B.STATS,          c = L.DRUID,                                                l = 62,                d = { s = 1126                         }                                                },
  { a = B.STATS,          c = L.HUNTER,       s = L.MARKSMAN,                         l = 100,  t = 21197,   d = { s = 160206, t = 21197            }, v = true                                      },
  { a = B.STATS,          c = L.HUNTER,       s = L.SURVIVAL,                         l = 100,  t = 21197,   d = { s = 160206, t = 21197            }, v = true                                      },
  { a = B.STATS,          c = L.HUNTER,                           p = P.DOG,                                 d = { s = 159988                       }                                                },
  { a = B.STATS,          c = L.HUNTER,                           p = P.GORILLA,                             d = { s = 160017                       }                                                },
  { a = B.STATS,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SHALESPIDER,                         d = { s = 90363                        }                                                },
  { a = B.STATS,          c = L.HUNTER,       s = L.BEASTMASTER,  p = P.WORM,                                d = { s = 160077                       }                                                },
  { a = B.STATS,          c = L.MONK,         s = L.MISTWEAVER,                       l = 22,                d = { s = 115921                       }                                                },
  { a = B.STATS,          c = L.MONK,         s = L.BREWMASTER,                       l = 81,                d = { s = 116781                       }                                                },
  { a = B.STATS,          c = L.MONK,         s = L.WINDWALKER,                       l = 81,                d = { s = 116781                       }                                                },
  { a = B.STATS,          c = L.PALADIN,                                              l = 30,                d = { s = 20217                        }                                                },

  { a = B.MASTERY,        c = L.DEATHKNIGHT,  s = L.BLOOD,                            l = 80,                d = { s = 155522                       }, v = true                                      },
  { a = B.MASTERY,        c = L.DRUID,        s = L.BALANCE,                          l = 16,                d = { s = 24858                        }                                                },
  { a = B.MASTERY,        c = L.PALADIN,                                              l = 81,                d = { s = 19740                        }                                                },
  { a = B.MASTERY,        c = L.SHAMAN,                                               l = 80,                d = { s = 116956                       }, v = true                                      },
  { a = B.MASTERY,        c = L.HUNTER,       s = L.MARKSMAN,                         l = 100,  t = 21197,   d = { s = 160198, t = 21197            }, v = true                                      },
  { a = B.MASTERY,        c = L.HUNTER,       s = L.SURVIVAL,                         l = 100,  t = 21197,   d = { s = 160198, t = 21197            }, v = true                                      },  
  { a = B.MASTERY,        c = L.HUNTER,                           p = P.CAT,          l = 81,                d = { s = 93435                        }                                                },
  { a = B.MASTERY,        c = L.HUNTER,                           p = P.HYDRA,        l = 81,                d = { s = 160039                       }                                                },
  { a = B.MASTERY,        c = L.HUNTER,                           p = P.TALLSTRIDER,  l = 81,                d = { s = 160073                       }                                                },
  { a = B.MASTERY,        c = L.HUNTER,       s = L.BEASTMASTER,  p = P.SPIRITBEAST,  l = 81,                d = { s = 128997                       }                                                },

  { a = C.FEARUNDEAD,     c = L.PALADIN,                                              l = 78,                d = { s = 10326                        }                                                }, -- Note: also included Demons!
  { a = C.FEARANY,        c = L.PRIEST ,                                              l = 60,   t = 19768,   d = {             t = 19768, i = 8122  }                                                },
  { a = C.FEARANY,        c = L.WARLOCK,                                              l = 14,                d = { s = 5782                         }                                                },
  { a = C.FEARANY,        c = L.WARLOCK,                                              l = 60,   t = 19290,   d = {             t = 19290, i = 111397}                                                },
  { a = C.FEARANY,        c = L.WARLOCK,                                              l = 30,   t = 19284,   d = {             t = 19284, i = 5484  }                                                },
  { a = C.FEARANY,        c = L.WARLOCK,                                              l = 30,   t = 19285,   d = {             t = 19285, i = 6789  }                                                },

  { a = M.INTERRUPT,      c = L.DEATHKNIGHT,                                          l = 57,                d = { s = 47528                        }                                                },
  { a = M.INTERRUPT,      c = L.DRUID,                                                l = 75,   t = 18581                                                                                            },
  { a = M.INTERRUPT,      c = L.DRUID,        s = L.FERAL,                            l = 64,                d = { s = 106839                       }                                                },
  { a = M.INTERRUPT,      c = L.DRUID,        s = L.GUARDIAN,                         l = 64,                d = { s = 106839                       }                                                },
  { a = M.INTERRUPT,      c = L.DRUID,        s = L.GUARDIAN,                                                d = { s = 770,    g = 114237           },                                   g = 114237  },
  { a = M.INTERRUPT,      c = L.HUNTER,                                               l = 22,                d = { s = 147362                       }                                                },
  { a = M.INTERRUPT,      c = L.MAGE,                                                 l = 9,                 d = { s = 2139                         }                                                },
  { a = M.INTERRUPT,      c = L.MAGE,                                                 l = 45,   t = 16021,   d = {             t = 16021, i = 102051}                                                },
  { a = M.INTERRUPT,      c = L.MONK,                                                 l = 32,                d = { s = 116705                       }                                                },
  { a = M.INTERRUPT,      c = L.PALADIN,                                              l = 54,                d = { s = 96231                        }                                                },
  { a = M.INTERRUPT,      c = L.PALADIN,      s = L.PROTPALADIN,                      l = 10,                d = { s = 31935                        }                                                },
  { a = M.INTERRUPT,      c = L.PRIEST,       s = L.SHADOW,                           l = 52,                d = { s = 15487,  g = 159626           },                                   g = 159626  },
  { a = M.INTERRUPT,      c = L.ROGUE,                                                l = 30,   t = 19236,   d = {             t = 19236, i = 26679 }                                                },
  { a = M.INTERRUPT,      c = L.ROGUE,                                                l = 14,                d = { s = 1766                         }                                                },
  { a = M.INTERRUPT,      c = L.SHAMAN,                                               l = 16,                d = { s = 57994                        }                                                },
  { a = M.INTERRUPT,      c = L.WARLOCK,                          p = P.FELHUNTER,                           d = { s = 19647                        }                                                },
  { a = M.INTERRUPT,      c = L.WARLOCK,                          p = P.DOOMGUARD,              t = 21182,   d = { s = 171138, t = 21182            }                                                },--Talentpet
  { a = M.INTERRUPT,      c = L.WARLOCK,                          p = P.OBSERVER,               t = 19293,   d = { s = 115781, t = 19293            }                                                },--Talentpet
  { a = M.INTERRUPT,      c = L.WARRIOR,                                              l = 24,                d = { s = 6552                         }                                                },

  { a = M.INTERRUPT2,     c = L.DEATHKNIGHT,                                          l = 58,                d = { s = 47476                        }                                                },
  { a = M.INTERRUPT2,     c = L.DEATHKNIGHT,                                          l = 58,   t = 19223                                                                                            },
  { a = M.INTERRUPT2,     c = L.DRUID,        s = L.BALANCE,                          l = 28,                d = { s = 78675                        }                                                },
  { a = M.INTERRUPT2,     c = L.PRIEST,       s = L.DISCIPLINE,                       l = 52,                d = { s = 15487                        }                                                },
  { a = M.INTERRUPT2,     c = L.PRIEST,       s = L.SHADOW,                           l = 52,                d = { s = 15487                        }                                                },

  { a = M.INTERRUPT2,                                                                                        d = { s = 25046                        }, display = L.BLOODELF_ROGUE,                   e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        d = { s = 28730                        }, display = L.BLOODELF_CASTER,                  e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        d = { s = 50613                        }, display = L.BLOODELF_DEATHKNIGHT,             e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        d = { s = 69179                        }, display = L.BLOODELF_WARRIOR,                 e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        d = { s = 80483                        }, display = L.BLOODELF_HUNTER,                  e = R.BLOODELF },
  { a = M.INTERRUPT2,                                                                                        d = { s = 129597                       }, display = L.BLOODELF_MONK,                    e = R.BLOODELF },

  { a = C.CCUNDEAD,       c = L.DEATHKNIGHT,                                          l = 69,                d = { s = 111673                       }                                                },

  { a = C.CCBEASTS,       c = L.MAGE,                                                 l = 14,                d = { s = 118                          }                                                },
  { a = C.CCHUMANOIDS,    c = L.MAGE,                                                 l = 14,                d = { s = 118                          }                                                },

  { a = C.CCDEMONS,       c = L.PALADIN,                                              l = 30,   t = 17575,   d = {             t = 17575, i = 20066 }                                                },
  { a = C.CCDRAGONKIN,    c = L.PALADIN,                                              l = 30,   t = 17575,   d = {             t = 17575, i = 20066 }                                                },
  { a = C.CCGIANTS,       c = L.PALADIN,                                              l = 30,   t = 17575,   d = {             t = 17575, i = 20066 }                                                },
  { a = C.CCHUMANOIDS,    c = L.PALADIN,                                              l = 30,   t = 17575,   d = {             t = 17575, i = 20066 }                                                },
  { a = C.CCUNDEAD,       c = L.PALADIN,                                              l = 30,   t = 17575,   d = {             t = 17575, i = 20066 }                                                },

  { a = C.CCUNDEAD,       c = L.PRIEST,                                               l = 32,                d = { s = 9484                         }                                                },
  { a = C.CCHUMANOIDS,    c = L.PRIEST,                                               l = 60,   t = 19755                                                                                            },

  { a = C.CCBEASTS,       c = L.ROGUE,                                                l = 10,                d = { s = 6770                         }                                                },
  { a = C.CCDEMONS,       c = L.ROGUE,                                                l = 10,                d = { s = 6770                         }                                                },
  { a = C.CCDRAGONKIN,    c = L.ROGUE,                                                l = 10,                d = { s = 6770                         }                                                },
  { a = C.CCHUMANOIDS,    c = L.ROGUE,                                                l = 10,                d = { s = 6770                         }                                                },

  { a = C.CCBEASTS,       c = L.SHAMAN,                                               l = 75,                d = { s = 51514                        }                                                },
  { a = C.CCHUMANOIDS,    c = L.SHAMAN,                                               l = 75,                d = { s = 51514                        }                                                },

  { a = C.CCBEASTS,       c = L.WARLOCK,                          p = P.SHIVARRA,               t = 19293,   d = { s = 115268, t = 19293            }                                                },--Talentpet
  { a = C.CCDEMONS,       c = L.WARLOCK,                                              l = 32,                d = { s = 710                          }                                                },
  { a = C.CCDEMONS,       c = L.WARLOCK,                                              l = 31,                d = { s = 1098                         }                                                },
  { a = C.CCDRAGONKIN,    c = L.WARLOCK,                          p = P.SHIVARRA,               t = 19293,   d = { s = 115268, t = 19293            }                                                },--Talentpet
  { a = C.CCGIANTS,       c = L.WARLOCK,                          p = P.SHIVARRA,               t = 19293,   d = { s = 115268, t = 19293            }                                                },--Talentpet
  { a = C.CCELEMENTALS,   c = L.WARLOCK,                                              l = 32,                d = { s = 710                          }                                                },
  { a = C.CCHUMANOIDS,    c = L.WARLOCK,                          p = P.SUCCUBUS,                            d = { s = 6358                         }                                                },
  { a = C.CCHUMANOIDS,    c = L.WARLOCK,                          p = P.SHIVARRA,               t = 19293,   d = { s = 115268, t = 19293            }                                                },--Talentpet
  { a = C.CCUNDEAD,       c = L.WARLOCK,                          p = P.SHIVARRA,               t = 19293,   d = { s = 115268, t = 19293            }                                                },--Talentpet

  { a = C.CCANY,          c = L.HUNTER,                                               l = 28,                d = { s = 1499                         }                                                },
  { a = C.CCANY,          c = L.HUNTER,                                               l = 30,   t = 19348,   d = {             t = 19348, i = 19386 }                                                },
  { a = C.CCANY,          c = L.MONK,                                                 l = 44,                d = { s = 115078                       }                                                },
  { a = C.CCANY,          c = L.ROGUE,                                                l = 38,                d = { s = 2094                         }                                                }, -- Maybe? 1 min disorient on a 2 min cooldown 

  { a = O.ROOT,           c = L.DRUID,                                                l = 10,                d = { s = 339                          }                                                },

  { a = O.SNARE,          c = L.DEATHKNIGHT,                                          l = 58,                d = { s = 45524                        }                                                },
  { a = O.SNARE,          c = L.DEATHKNIGHT,                                          l = 58,   t = 19222,   d = { s = 55095,  t = 19222            }                                                },
  { a = O.SNARE,          c = L.DRUID,        s = L.BALANCE,                          l = 30,   t = 21776,   d = { s = 102355                       }                                                },
  { a = O.SNARE,          c = L.DRUID,        s = L.RESTODRUID,                       l = 30,   t = 21778,   d = { s = 102355                       }                                                },
  { a = O.SNARE,          c = L.DRUID,        s = L.FERAL,                            l = 30,   t = 18575,   d = { s = 106707                       }                                                },
  { a = O.SNARE,          c = L.DRUID,        s = L.GUARDIAN,                         l = 30,   t = 21777,   d = { s = 106707                       }                                                },
  { a = O.SNARE,          c = L.DRUID,        s = L.FERAL,                            l = 40,                d = { s = 48484                        }                                                },
  { a = O.SNARE,          c = L.DRUID,        s = L.GUARDIAN,                         l = 40,                d = { s = 48484                        }                                                },
  { a = O.SNARE,          c = L.HUNTER,                                               l = 8,                 d = { s = 5116                         }                                                },
  { a = O.SNARE,          c = L.HUNTER,                                               l = 100,  t = 21195,   d = { s = 162539, t = 21195            }                                                },
  { a = O.SNARE,          c = L.MAGE,                                                 l = 1,                 d = { s = 44614                        }                                                },
  { a = O.SNARE,          c = L.MAGE,         s = L.FROST,                            l = 12,                d = { s = 116                          }                                                },
  { a = O.SNARE,          c = L.MAGE,         s = L.ARCANE,                           l = 36,                d = { s = 31589                        }                                                },
  { a = O.SNARE,          c = L.MAGE,         s = L.ARCANE,                           l = 75,                d = { s = 30451,  g = 86209            },                                   g = 86209   },
  { a = O.SNARE,          c = L.MONK,         s = L.WINDWALKER,                       l = 28,                d = { s = 116095                       }                                                },
  { a = O.SNARE,          c = L.PALADIN,      s = L.RETRIBUTION,                      l = 70,                d = { s = 20164                        }                                                },
  { a = O.SNARE,          c = L.PRIEST,       s = L.SHADOW,                           l = 10,                d = { s = 15407                        }                                                },
  { a = O.SNARE,          c = L.ROGUE,                                                l = 20,                d = { s = 3408                         }                                                },
  { a = O.SNARE,          c = L.SHAMAN,                                               l = 22,                d = { s = 8056                         }                                                }, -- Frostshock!
  { a = O.SNARE,          c = L.SHAMAN,                                                                                                                                                  g = 161654  },
  { a = O.SNARE,          c = L.WARRIOR,                                              l = 36,                d = { s = 1715                         }                                                },
  { a = O.SNARE,          c = L.WARRIOR,      s = L.PROTWARRIOR,                      l = 25,                d = { s = 78,     g = 58366            },                                   g = 58366   },

  { a = O.AOESNARE,       c = L.DRUID,        s = L.BALANCE,                          l = 42,                d = { s = 16914,  g = 54831            },                                   g = 54831   },
  { a = O.AOESNARE,       c = L.HUNTER,                                               l = 46,                d = { s = 13809                        }                                                },
  { a = O.AOESNARE,       c = L.MAGE,                                                 l = 52,                d = { s = 10                           }                                                },
  { a = O.AOESNARE,       c = L.MAGE,                                                 l = 100,               d = { s = 2120,   k = 157621           }                                                }, -- perk, Improved Flamestrike, possible at 92+
  { a = O.AOESNARE,       c = L.MONK,         s = L.BREWMASTER,                       l = 10,                d = { s = 115180                       }                                                },
  { a = O.AOESNARE,       c = L.MONK,         s = L.BREWMASTER,                       l = 11,                d = { s = 121253                       }                                                },
  { a = O.AOESNARE,       c = L.WARRIOR,                                              l = 15,                d = { s = 12323                        }                                                },

  { a = O.STUN,           c = L.DEATHKNIGHT,                                          l = 58,   t = 19223                                                                                            },
  { a = O.STUN,           c = L.DEATHKNIGHT,                      p = P.GHOUL,                               d = { s = 47481                        }                                                },
  { a = O.STUN,           c = L.DRUID,                                                l = 75,   t = 18581,   d = {             t = 18581, i = 99    }                                                },
  { a = O.STUN,           c = L.DRUID,                                                l = 75,   t = 18583,   d = {             t = 18583, i = 5211  }                                                },
  { a = O.STUN,           c = L.DRUID,        s = L.FERAL,                            l = 82,                d = { s = 22570                        }                                                },
  { a = O.STUN,           c = L.HUNTER,                                               l = 30,   t = 19359,   d = {             t = 19359, i = 19577 }                                                },
  { a = O.STUN,           c = L.PALADIN,                                              l = 7,                 d = { s = 853                          }                                                },
  { a = O.STUN,           c = L.PALADIN,                                              l = 30,   t = 17573,   d = {             t = 17573, i = 105593}                                                },
  { a = O.STUN,           c = L.ROGUE,                                                l = 30,                d = { s = 1833                         }                                                },
  { a = O.STUN,           c = L.ROGUE,                                                l = 40,                d = { s = 408                          }                                                },
  { a = O.STUN,           c = L.WARRIOR,                                              l = 60,   t = 15759,   d = {             t = 15759, i = 107570}                                                },

  { a = O.AOESTUN,        c = L.DEATHKNIGHT,                                          l = 50,   t = 19231                                                                                            },
  { a = O.AOESTUN,        c = L.HUNTER,                                               l = 30,   t = 19347,   d = {             t = 19347, i = 109248}                                                },
  { a = O.AOESTUN,        c = L.MAGE,                                                 l = 45,   t = 16019,   d = {             t = 16019, i = 113724}                                                },
  { a = O.AOESTUN,        c = L.MONK,                                                 l = 60,   t = 19994,   d = {             t = 19994, i = 119392}                                                },
  { a = O.AOESTUN,        c = L.MONK,                                                 l = 60,   t = 19995,   d = {             t = 19995, i = 119381}                                                },
  { a = O.AOESTUN,        c = L.MONK,         s = L.WINDWALKER,                       l = 10,                d = { s = 113656                       }                                                },
  { a = O.AOESTUN,        c = L.SHAMAN,                                               l = 63,                d = { s = 108269                       }                                                },
  { a = O.AOESTUN,        c = L.WARLOCK,                                              l = 35,   t = 19286                                                                                            },
  { a = O.AOESTUN,        c = L.WARLOCK,                          p = P.INFERNAL,               t = 21182,   d = { s = 171018, t = 21182            }                                                },
  { a = O.AOESTUN,        c = L.WARRIOR,                                              l = 60,   t = 15760,   d = {             t = 15760, i = 46968 }                                                },

  { a = M.MORTALSTRIKE,   c = L.HUNTER,                           p = P.CROCOLISK,                           d = { s = 159936                       }                                                },
  { a = M.MORTALSTRIKE,   c = L.HUNTER,                           p = P.CARRIONBIRD,                         d = { s = 24423                        }                                                },
  { a = M.MORTALSTRIKE,   c = L.HUNTER,                           p = P.RIVERBEAST,                          d = { s = 160018                       }                                                },
  { a = M.MORTALSTRIKE,   c = L.HUNTER,                           p = P.SCORPID,                             d = { s = 160060                       }                                                },
  { a = M.MORTALSTRIKE,   c = L.HUNTER,       s = L.BEASTMASTER,  p = P.DEVILSAUR,                           d = { s = 54680                        }                                                },
  { a = M.MORTALSTRIKE,   c = L.MONK,         s = L.MISTWEAVER,                       l = 56,                d = { s = 107428                       }                                                },
  { a = M.MORTALSTRIKE,   c = L.MONK,         s = L.WINDWALKER,                       l = 56,                d = { s = 107428                       }                                                },
  { a = M.MORTALSTRIKE,   c = L.ROGUE,                                                l = 30,                d = { s = 8679                         }                                                },
  { a = M.MORTALSTRIKE,   c = L.WARRIOR,      s = L.ARMS,                             l = 10,                d = { s = 12294                        }                                                },
  { a = M.MORTALSTRIKE,   c = L.WARRIOR,      s = L.FURY,                             l = 18,                d = { s = 100130                       }                                                },
  { a = M.MORTALSTRIKE,   c = L.WARLOCK,                          p = P.FELGUARD,                            d = { s = 30213                        }                                                },
  { a = M.MORTALSTRIKE,   c = L.WARLOCK,                          p = P.WRATHGUARD,             t = 19293,   d = { s = 115625, t = 19293            }                                                },--Talentpet

  { a = O.KNOCKBACK,      c = L.DRUID,                                                l = 45,   t = 18577,   d = {             t = 18577, i = 132469}                                                },
  { a = O.KNOCKBACK,      c = L.HUNTER,                                               l = 100,  t = 19358,   d = {             t = 19358, i = 109259}                                                },
  { a = O.KNOCKBACK,      c = L.HUNTER,                                                                      d = { s = 13813,  g = 119403           },                                g = 119403     },
  { a = O.KNOCKBACK,      c = L.SHAMAN,       s = L.ELEMENTAL,                        l = 10,                d = { s = 51490                        }                                                },
  { a = O.KNOCKBACK,      c = L.WARLOCK,                          p = P.SUCCUBUS,                            d = { s = 6360                         }                                                },
  { a = O.KNOCKBACK,      c = L.WARLOCK,                          p = P.SHIVARRA,               t = 19293,   d = { s = 115770, t = 19293            }                                                },--Talentpet
  { a = O.KNOCKBACK,      c = L.WARRIOR,      s = L.ARMS,                             l = 100,  t = 21760,   d = {             t = 21760, i = 176289}                                                },
  { a = O.KNOCKBACK,      c = L.WARRIOR,      s = L.FURY,                             l = 100,  t = 21760,   d = {             t = 21760, i = 176289}                                                },
}

-- Tooltip display text
for _, entry in ipairs (data.table) do
  -- Common display format unless overridden in the table above
  if not entry.display then 
    local disp = entry.c
    if entry.s then disp = disp..SPEC..entry.s end
    if entry.p then disp = disp..L.PET..entry.p end
    if entry.g then disp = disp..L.GLYPH end
    if entry.t then disp = disp..L.TOOLTIP_TALENT end
    if entry.v then disp = disp..L.TOOLTIP_PASSIVE end
    entry.display = disp
  end
end

